package RebuildIndexFilter::CMS;

use strict;
use warnings;

use MT::RebuildIndexFilter;

our $schedule_entry = 0;
our $schedule_page = 0;

sub rebuild_index_filter {
    my ($eh, %args) = @_;
    return 1 if $args{'ArchiveType'} ne 'index';

    my $id = $args{template}->id;
    my $rif = MT::RebuildIndexFilter->load({ template_id => $id });

    # no setting
    return 1 unless (defined $rif);

    my $rebuild_event = $rif->rebuild_event;

    # no check or all
    return 1 if ($rebuild_event eq '' || $rebuild_event eq 'all');

    my $script = $ENV{'SCRIPT_NAME'};

    my $type;
    my $app = MT->instance;
    my $admin_script = $app->config('AdminScript');

    if ((defined $admin_script) && ($script =~ /$admin_script/)) {
        my $q = $app->param;

        # rebuild
        if ($q->param('__mode') eq 'rebuild') {

            # on template page
            return 1 if ($q->param('return_args') =~ /_type=template/);

            # on rebuild display
            return 1 if ($q->param('type') =~ /index/)
        }

        # operation on list_comment
        if ($q->param('return_args') =~ /__mode=list_comment/) {
            return 1 if ($q->param('return_args') =~ /approved=1|junked=1/) && ($rebuild_event =~ /comment/);
        }

        # operation on list_pings
        if ($q->param('return_args') =~ /__mode=list_pings/) {
            return 1 if ($q->param('return_args') =~ /approved=1|junked=1/) && ($rebuild_event =~ /ping/);
        }

        # operation on list_entry
        return 1 if ($q->param('return_args') =~ /__mode=list_entry/) && ($rebuild_event =~ /entry/);

        # operation on list_pages
        return 1 if ($q->param('return_args') =~ /__mode=list_pages/) && ($rebuild_event =~ /page/);

        $type = $q->param('_type');

        # operation on comment edit page.
        if ($type eq 'comment') {
            return 1 if $rebuild_event =~ /comment/;
        }

        # operation on ping edit page.
        if ($type eq 'ping') {
            return 1 if $rebuild_event =~ /ping/;
        }

        # operation on mail
        if ($q->param('return_args') =~ /_type=comment/) {
            return 1 if ($q->param('return_args') =~ /approved=1|junked=1/) && ($rebuild_event =~ /comment/);
        }

        # operation on mail
        if ($q->param('return_args') =~ /_type=ping/) {
            return 1 if ($q->param('return_args') =~ /approved=1|junked=1/) && ($rebuild_event =~ /ping/);
        }

        # edit entry or page
        $type = '';
        if ($q->param('return_args') eq '') {
            my $entry = MT::Entry->load($q->param('entry_id'));
            if ($entry) {
                $type = $entry->class;
            }
        } else {
            # new entry or page
            $type = $q->param('_type');
        }
        if ($type) {
            my @event = ('entry','page');
            for my $event (@event) {
                return 1 if ($event =~ /$type/) && ($rebuild_event =~ /$event/);
            }
        }
    }

    # post comment
    my $comment_script = $app->config('CommentScript');
    return 1 if (defined $comment_script) && ($script =~ /$comment_script/) && ($rebuild_event =~ /comment/);

    # receive ping
    my $trackback_script = $app->config('TrackbackScript');
    return 1 if (defined $trackback_script) && ($script =~ /$trackback_script/) && ($rebuild_event =~ /ping/);

    # schedule task(perhaps)
    my $match;
    if (!$script) {
        $match = 1 if ($schedule_entry && $rebuild_event =~ /entry/) || ($schedule_page && $rebuild_event =~ /page/);
    }
    return 1 if $match;

    return 0;
}

sub get_schedule {
    my ($eh, $mt, $entry) = @_;
    if ($entry) {
        my $type = $entry->class;
        $schedule_entry = 1 if ($type eq 'entry');
        $schedule_page = 1 if ($type eq 'page');
    }
    return 1;
}

sub add_rebuild_event {
    my ($cb, $app, $template) = @_;

    my $old = <<HERE;
    </mt:if>
</mt:if>
<mt:if name="template_group" like="(module|widget)">
HERE
    $old = quotemeta($old);

    my $label = MT->translate("Rebuild_event");
    my $all = MT->translate("All");
    my $entry = MT->translate("Entry");
    my $page = MT->translate("Page");
    my $comment = MT->translate("Comment");
    my $ping = MT->translate("Ping");

    my $new = <<HERE;
<script type="text/javascript">
function toggleAndUncheckForRebuildEvent(obj, target) {
    if (!obj || !target) {
        return;
    }
    var targetObj = getByID(target);
    if (!targetObj) {
        return;
    }
    if (obj.checked) {
//        targetObj.checked = true;
        targetObj.disabled = true;
    } else {
//        targetObj.checked = false;
        targetObj.disabled = false;
    }
}
function checkEventsForRebuildEvent(obj) {
    toggleAndUncheckForRebuildEvent(obj,'rebuild-event-entry');
    toggleAndUncheckForRebuildEvent(obj,'rebuild-event-page');
    toggleAndUncheckForRebuildEvent(obj,'rebuild-event-comment');
    toggleAndUncheckForRebuildEvent(obj,'rebuild-event-ping');
}
</script>
        <mtapp:setting
            id="rebuild_event"
            label="$label"
            content_class="field-content-text">
            <ul id="rebuild-events">
                <li><input type="checkbox" value="all" name="rebuild_event" id="rebuild-event-all" onclick="checkEventsForRebuildEvent(this)"<mt:if name="rebuild_event" eq="all"> checked="checked"</mt:if>> <label for="cache-event-all">$all</label></li>
                <li><input type="checkbox" value="entry" name="rebuild_event" id="rebuild-event-entry" <mt:if name="rebuild_event" like="entry"> checked="checked"</mt:if><mt:if name="rebuild_event" like="all"> disabled="disabled"</mt:if>> <label for="rebuild-event-entry">$entry</label></li>
                <li><input type="checkbox" value="page" name="rebuild_event" id="rebuild-event-page" <mt:if name="rebuild_event" like="page"> checked="checked"</mt:if><mt:if name="rebuild_event" like="all"> disabled="disabled"</mt:if>> <label for="rebuild-event-page">$page</label></li>
                <li><input type="checkbox" value="comment" name="rebuild_event" id="rebuild-event-comment" <mt:if name="rebuild_event" like="comment"> checked="checked"</mt:if><mt:if name="rebuild_event" like="all"> disabled="disabled"</mt:if>> <label for="rebuild-event-comment">$comment</label></li>
                <li><input type="checkbox" value="ping" name="rebuild_event" id="rebuild-event-ping" <mt:if name="rebuild_event" like="ping"> checked="checked"</mt:if><mt:if name="rebuild_event" like="all"> disabled="disabled"</mt:if>> <label for="rebuild-event-ping">$ping</label></li>
            </ul>
        </mtapp:setting>
    </mt:if>
</mt:if>
<style type="text/css">
#rebuild_event-field .field-header { width: 7em; }
#rebuild_event-field .field-content { margin-left: 8em; }
</style>
<mt:if name="template_group" like="(module|widget)">
HERE
    $$template =~ s/$old/$new/;
}

sub save_rebuild_event {
    my ($cb, $app, $obj, $org) = @_;
    my $id = $obj->id;
    if ($id) {
        my $rif = MT::RebuildIndexFilter->load({ template_id => $id });
        my $q = $app->param;
        my @rebuild_event = $q->param('rebuild_event');
        if (@rebuild_event) {
            unless (defined $rif) {
                $rif = MT::RebuildIndexFilter->new;
                $rif->template_id($id);
            }
            $rif->rebuild_event(join(',', @rebuild_event));
            $rif->save or die $rif->errstr;
        } else {
            if (defined $rif) {
                $rif->rebuild_event('');
                $rif->save or die $rif->errstr;
            }
        }
    }
    return 1;
}

sub delete_rebuild_event {
    my ($cb, $app, $obj) = @_;
    my $id = $obj->id;
    if ($id) {
        my $rif = MT::RebuildIndexFilter->load({ template_id => $id });
        return 1 unless defined $rif;
        $rif->remove;
    }
    return 1;
}

sub get_rebuild_event {
    my ($cb, $app, $param, $tmpl) = @_;
    my $id = $param->{id};
    if ($id) {
        my $rif = MT::RebuildIndexFilter->load({ template_id => $id });
        unless (defined $rif) {
            $param->{rebuild_event} = '';
        } else {
            $param->{rebuild_event} = $rif->rebuild_event;
        }
    }
}

1;
